@@ -22,6 +22,7 @@ from typing import (
 )
 
 from debputy import DEBPUTY_ROOT_DIR, DEBPUTY_PLUGIN_ROOT_DIR
+from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
 from debputy.analysis import REFERENCE_DATA_TABLE
 from debputy.analysis.debian_dir import scan_debian_dir
 from debputy.build_support import perform_clean, perform_builds
@@ -31,7 +32,7 @@ from debputy.commands.debputy_cmd.context import (
     ROOT_COMMAND,
     CommandArg,
 )
-from debputy.commands.debputy_cmd.output import _stream_to_pager
+from debputy.commands.debputy_cmd.output import _stream_to_pager, _output_styling
 from debputy.dh_migration.migrators import MIGRATORS
 from debputy.exceptions import (
     DebputyRuntimeError,
@@ -770,9 +771,7 @@ def assemble(
     source_version = manifest.source_version()
     is_native = "-" not in source_version
     is_dh_rrr_only_mode = integration_mode == INTEGRATION_MODE_DH_DEBPUTY_RRR
-    package_data_table = manifest.perform_installations(
-        enable_manifest_installation_feature=not is_dh_rrr_only_mode
-    )
+    package_data_table = manifest.perform_installations(integration_mode)
     if not is_dh_rrr_only_mode:
         for dctrl_bin in manifest.active_packages:
             package = dctrl_bin.name
@@ -1003,6 +1002,7 @@ def _migrate_from_dh(context: CommandContext) -> None:
         )
     )
     migrate_from_dh(
+        _output_styling(context.parsed_args, sys.stdout),
         manifest,
         acceptable_migration_issues,
         parsed_args.destructive,
