@@ -24,6 +24,7 @@ from debian.deb822 import Deb822
 
 from debputy import DEBPUTY_DOC_ROOT_DIR
 from debputy.architecture_support import dpkg_architecture_table
+from debputy.commands.debputy_cmd.output import OutputStylingBase
 from debputy.deb_packaging_support import dpkg_field_list_pkg_dep
 from debputy.dh.debhelper_emulation import (
     dhe_filedoublearray,
@@ -1500,16 +1501,18 @@ def detect_obsolete_substvars(
                 seen_obsolete_relationship_substvars.update(obsolete_substvars_in_field)
 
         package = p.get("Package", "(Missing package name!?)")
+        fo = feature_migration.fo
         if obsolete_fields:
             fields = ", ".join(obsolete_fields)
             feature_migration.warn(
                 f"The following relationship fields can be removed from {package}: {fields}."
-                f"  (The content in them would be applied automatically.)"
+                f"  (The content in them would be applied automatically. Note: {fo.bts('1067653')})"
             )
         if seen_obsolete_relationship_substvars:
             v = ", ".join(sorted(seen_obsolete_relationship_substvars))
             feature_migration.warn(
                 f"The following relationship substitution variables can be removed from {package}: {v}"
+                f" (Note: {fo.bts('1067653')})"
             )
 
 
@@ -1552,6 +1555,9 @@ def detect_dh_addons_with_full_integration(
     feature_migration.warn(
         "TODO: Not implemented: Please ensure there is a Build-Dependency on `debputy (>= 0.1.45~)"
     )
+    feature_migration.warn(
+        "TODO: Not implemented: Please ensure there is a Build-Dependency on `dpkg-dev (>= 1.22.7~)"
+    )
 
 
 def detect_dh_addons_with_zz_integration(
