@@ -1,5 +1,6 @@
 import dataclasses
 import os
+import time
 from typing import (
     Optional,
     List,
@@ -17,19 +18,19 @@ from debputy.dh.dh_assistant import (
     DhSequencerData,
     extract_dh_addons_from_control,
 )
-from debputy.lsprotocol.types import MarkupKind
-
 from debputy.filesystem_scan import FSROOverlay, VirtualPathBase
 from debputy.linting.lint_util import (
     LintState,
 )
-from debputy.lsp.style_prefs import (
-    StylePreferenceTable,
+from debputy.lsp.apt_cache import AptCache
+from debputy.lsp.maint_prefs import (
+    MaintainerPreferenceTable,
     MaintainerPreference,
-    determine_effective_style,
+    determine_effective_preference,
 )
 from debputy.lsp.text_util import LintCapablePositionCodec
 from debputy.lsp.vendoring._deb822_repro import Deb822FileElement, parse_deb822_file
+from debputy.lsprotocol.types import MarkupKind
 from debputy.packages import (
     SourcePackage,
     BinaryPackage,
@@ -301,16 +302,16 @@ class LSProvidedLintState(LintState):
         salsa_ci = self._resolve_salsa_ci()
         if source_package is None and salsa_ci is None:
             return None
-        style, _ = determine_effective_style(
-            self.style_preference_table,
+        style, _, _ = determine_effective_preference(
+            self.maint_preference_table,
             source_package,
             salsa_ci,
         )
         return style
 
     @property
-    def style_preference_table(self) -> StylePreferenceTable:
-        return self._ls.style_preferences
+    def maint_preference_table(self) -> MaintainerPreferenceTable:
+        return self._ls.maint_preferences
 
     @property
     def salsa_ci(self) -> Optional[CommentedMap]:
@@ -360,20 +361,42 @@ class DebputyLanguageServer(LanguageServer):
         self._plugin_feature_set: Optional[PluginProvidedFeatureSet] = None
         self._trust_language_ids: Optional[bool] = None
         self._finished_initialization = False
-        self.style_preferences = StylePreferenceTable({}, {})
+        self.maint_preferences = MaintainerPreferenceTable({}, {})
+        self.apt_cache = AptCache()
+        self.background_tasks = set()
 
-    def finish_initialization(self) -> None:
+    def finish_startup_initialization(self) -> None:
         if self._finished_initialization:
             return
         assert self._dctrl_parser is not None
         assert self._plugin_feature_set is not None
         assert self._trust_language_ids is not None
-        self.style_preferences = self.style_preferences.load_styles()
+        self.maint_preferences = self.maint_preferences.load_preferences()
         _info(
-            f"Loaded style preferences: {len(self.style_preferences.maintainer_preferences)} unique maintainer preferences recorded"
+            f"Loaded style preferences: {len(self.maint_preferences.maintainer_preferences)} unique maintainer preferences recorded"
         )
         self._finished_initialization = True
 
+    async def on_initialize(self) -> None:
+        task = self.loop.create_task(self._load_apt_cache(), name="Index apt cache")
+        self.background_tasks.add(task)
+        task.add_done_callback(self.background_tasks.discard)
+
+    def shutdown(self) -> None:
+        for task in self.background_tasks:
+            _info(f"Cancelling task: {task.get_name()}")
+            self.loop.call_soon_threadsafe(task.cancel)
+        return super().shutdown()
+
+    async def _load_apt_cache(self) -> None:
+        _info("Starting load of apt cache data")
+        start = time.time()
+        await self.apt_cache.load()
+        end = time.time()
+        _info(
+            f"Loading apt cache finished after {end-start} seconds and is now in state {self.apt_cache.state}"
+        )
+
     @property
     def plugin_feature_set(self) -> PluginProvidedFeatureSet:
         res = self._plugin_feature_set
