@@ -25,6 +25,7 @@ from typing import (
     Tuple,
     get_args,
     Container,
+    final,
 )
 
 from debian.substvars import Substvars
@@ -32,17 +33,23 @@ from debian.substvars import Substvars
 from debputy import util
 from debputy.exceptions import TestPathWithNonExistentFSPathError, PureVirtualPathError
 from debputy.interpreter import Interpreter, extract_shebang_interpreter_from_file
+from debputy.manifest_parser.tagging_types import DebputyDispatchableType
 from debputy.manifest_parser.util import parse_symbolic_mode
 from debputy.packages import BinaryPackage
 from debputy.types import S
 
 if TYPE_CHECKING:
+    from debputy.plugin.debputy.to_be_api_types import BuildRule, BSR, BuildSystemRule
+    from debputy.plugin.api.impl_types import DIPHandler
     from debputy.manifest_parser.base_types import (
         StaticFileSystemOwner,
         StaticFileSystemGroup,
     )
 
 
+DP = TypeVar("DP", bound=DebputyDispatchableType)
+
+
 PluginInitializationEntryPoint = Callable[["DebputyPluginInitializer"], None]
 MetadataAutoDetector = Callable[
     ["VirtualPath", "BinaryCtrlAccessor", "PackageProcessingContext"], None
@@ -86,17 +93,20 @@ DebputyIntegrationMode = Literal[
     "dh-sequence-zz-debputy-rrr",
 ]
 
+INTEGRATION_MODE_FULL: DebputyIntegrationMode = "full"
 INTEGRATION_MODE_DH_DEBPUTY_RRR: DebputyIntegrationMode = "dh-sequence-zz-debputy-rrr"
 INTEGRATION_MODE_DH_DEBPUTY: DebputyIntegrationMode = "dh-sequence-zz-debputy"
 ALL_DEBPUTY_INTEGRATION_MODES: FrozenSet[DebputyIntegrationMode] = frozenset(
     get_args(DebputyIntegrationMode)
 )
 
+_DEBPUTY_DISPATCH_METADATA_ATTR_NAME = "_debputy_dispatch_metadata"
+
 
 def only_integrations(
     *integrations: DebputyIntegrationMode,
 ) -> Container[DebputyIntegrationMode]:
-    return frozenset(*integrations)
+    return frozenset(integrations)
 
 
 def not_integrations(
@@ -212,6 +222,27 @@ class PathDef:
     materialized_content: Optional[str] = None
 
 
+@dataclasses.dataclass(slots=True, frozen=True)
+class DispatchablePluggableManifestRuleMetadata(Generic[DP]):
+    """NOT PUBLIC API (used internally by part of the public API)"""
+
+    manifest_keywords: Sequence[str]
+    dispatched_type: Type[DP]
+    unwrapped_constructor: "DIPHandler"
+    expected_debputy_integration_mode: Optional[Container[DebputyIntegrationMode]] = (
+        None
+    )
+    online_reference_documentation: Optional["ParserDocumentation"] = None
+    apply_standard_attribute_documentation: bool = False
+    source_format: Optional[Any] = None
+
+
+@dataclasses.dataclass(slots=True, frozen=True)
+class BuildSystemManifestRuleMetadata(DispatchablePluggableManifestRuleMetadata):
+    build_system_impl: Optional[Type["BuildSystemRule"]] = (None,)
+    auto_detection_shadow_build_systems: FrozenSet[str] = frozenset()
+
+
 def virtual_path_def(
     path_name: str,
     /,
@@ -1507,6 +1538,16 @@ class ParserAttributeDocumentation:
     attributes: FrozenSet[str]
     description: Optional[str]
 
+    @property
+    def is_hidden(self) -> bool:
+        return False
+
+
+@final
+@dataclasses.dataclass(slots=True, frozen=True)
+class StandardParserAttributeDocumentation(ParserAttributeDocumentation):
+    sort_category: int = 0
+
 
 def undocumented_attr(attr: str) -> ParserAttributeDocumentation:
     """Describe an attribute as undocumented
@@ -1514,6 +1555,8 @@ def undocumented_attr(attr: str) -> ParserAttributeDocumentation:
     If you for some reason do not want to document a particular attribute, you can mark it as
     undocumented. This is required if you are only documenting a subset of the attributes,
     because `debputy` assumes any omission to be a mistake.
+
+    :param attr: Name of the attribute
     """
     return ParserAttributeDocumentation(
         frozenset({attr}),
