@@ -17,14 +17,10 @@ from typing import (
 
 from debputy import DEBPUTY_DOC_ROOT_DIR
 from debputy.maintscript_snippet import DpkgMaintscriptHelperCommand, MaintscriptSnippet
-from debputy.manifest_parser.base_types import (
-    DebputyParsedContent,
-    FileSystemExactMatchRule,
-)
-from debputy.manifest_parser.declarative_parser import (
-    DebputyParseHint,
-    ParserGenerator,
-)
+from debputy.manifest_parser.base_types import FileSystemExactMatchRule
+from debputy.manifest_parser.tagging_types import DebputyParsedContent
+from debputy.manifest_parser.parse_hints import DebputyParseHint
+from debputy.manifest_parser.declarative_parser import ParserGenerator
 from debputy.manifest_parser.exceptions import ManifestParseException
 from debputy.manifest_parser.parser_data import ParserContextData
 from debputy.manifest_parser.util import AttributePath
@@ -34,7 +30,7 @@ from debputy.plugin.api.impl import (
     DebputyPluginInitializerProvider,
     ServiceDefinitionImpl,
 )
-from debputy.plugin.api.impl_types import OPARSER_PACKAGES
+from debputy.plugin.api.parser_tables import OPARSER_PACKAGES
 from debputy.plugin.api.spec import (
     ServiceUpgradeRule,
     ServiceDefinition,
