@@ -657,9 +657,7 @@ class PerlBuildBuildSystemRule(StepBasedBuildSystemRule):
                         perl5lib_dir + perl_config_data.path_sep + env_perl5lib
                     )
                 env_mod = EnvironmentModification(
-                    replacements=[
-                        ("PERL5LIB", perl5lib_dir),
-                    ],
+                    replacements=(("PERL5LIB", perl5lib_dir),),
                 )
                 return perl_config_data, env_mod
         return perl_config_data, None
@@ -672,7 +670,10 @@ class PerlBuildBuildSystemRule(StepBasedBuildSystemRule):
     ) -> None:
         perl_config_data, cross_env_mod = self._perl_cross_build_env(context)
         configure_env = EnvironmentModification(
-            replacements=[("PERL_MM_USE_DEFAULT", "1")]
+            replacements=(
+                ("PERL_MM_USE_DEFAULT", "1"),
+                ("PKG_CONFIG", context.cross_tool("pkg-config")),
+            )
         )
         if cross_env_mod is not None:
             configure_env = configure_env.combine(cross_env_mod)
@@ -804,10 +805,11 @@ class PerlMakeMakerBuildSystemRule(StepBasedBuildSystemRule):
         **kwargs,
     ) -> None:
         configure_env = EnvironmentModification(
-            replacements=[
+            replacements=(
                 ("PERL_MM_USE_DEFAULT", "1"),
                 ("PERL_AUTOINSTALL", "--skipdeps"),
-            ]
+                ("PKG_CONFIG", context.cross_tool("pkg-config")),
+            )
         )
         perl_args = []
         mm_args = ["INSTALLDIRS=vendor"]
@@ -931,6 +933,8 @@ class DebhelperBuildSystemRule(StepBasedBuildSystemRule):
         try:
             v = subprocess.check_output(
                 ["dh_assistant", "which-build-system"],
+                # Packages without `debhelper-compat` will trigger an error, which will just be noise
+                stderr=subprocess.DEVNULL,
                 cwd=source_root.fs_path,
             )
         except subprocess.CalledProcessError:
@@ -1255,7 +1259,9 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
             replacements["DEB_PYTHON_INSTALL_LAYOUT"] = "deb"
         if "PKG_CONFIG" not in os.environ:
             replacements["PKG_CONFIG"] = build_context.cross_tool("pkg-config")
-        return EnvironmentModification(replacements=replacements)
+        return EnvironmentModification(
+            replacements=tuple((k, v) for k, v in replacements.items())
+        )
 
     @classmethod
     def cmake_generator(cls, target_build_system: Literal["make", "ninja"]) -> str:
@@ -1361,19 +1367,17 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
             env_mod = env_mod.combine(
                 # The debhelper build system never showed this delta, so people might find it annoying.
                 EnvironmentModification(
-                    replacements={
-                        "CFLAGS": cflags,
-                        "CXXFLAGS": cxxflags,
-                    }
+                    replacements=(
+                        ("CFLAGS", cflags),
+                        ("CXXFLAGS", cxxflags),
+                    )
                 )
             )
         if "ASMFLAGS" not in os.environ and "ASFLAGS" in os.environ:
             env_mod = env_mod.combine(
                 # The debhelper build system never showed this delta, so people might find it annoying.
                 EnvironmentModification(
-                    replacements={
-                        "ASMFLAGS": os.environ["ASFLAGS"],
-                    }
+                    replacements=(("ASMFLAGS", os.environ["ASFLAGS"]),),
                 )
             )
         self.ensure_build_dir_exists()
@@ -1413,9 +1417,7 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
         **kwargs,
     ) -> None:
         env_mod = EnvironmentModification(
-            replacements={
-                "CTEST_OUTPUT_ON_FAILURE": "1",
-            },
+            replacements=(("CTEST_OUTPUT_ON_FAILURE", "1"),),
         )
         if self.target_build_system == "make":
             # Unlike make, CTest does not have "unlimited parallel" setting (-j implies
@@ -1453,10 +1455,10 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
         **kwargs,
     ) -> None:
         env_mod = EnvironmentModification(
-            replacements={
-                "LC_ALL": "C.UTF-8",
-                "DESTDIR": dest_dir,
-            }
+            replacements=(
+                ("LC_ALL", "C.UTF-8"),
+                ("DESTDIR", dest_dir),
+            )
         ).combine(self._default_cmake_env(context))
         run_build_system_command(
             "cmake",
@@ -1524,7 +1526,9 @@ class MesonBuildSystemRule(StepBasedBuildSystemRule):
         }
         if "DEB_PYTHON_INSTALL_LAYOUT" not in os.environ:
             replacements["DEB_PYTHON_INSTALL_LAYOUT"] = "deb"
-        return EnvironmentModification(replacements=replacements)
+        return EnvironmentModification(
+            replacements=tuple((k, v) for k, v in replacements.items())
+        )
 
     @classmethod
     def cmake_generator(cls, target_build_system: Literal["make", "ninja"]) -> str:
@@ -1638,9 +1642,7 @@ class MesonBuildSystemRule(StepBasedBuildSystemRule):
         **kwargs,
     ) -> None:
         env_mod = EnvironmentModification(
-            replacements={
-                "MESON_TESTTHREDS": f"{context.parallelization_limit()}",
-            },
+            replacements=(("MESON_TESTTHREDS", f"{context.parallelization_limit()}"),),
         ).combine(self._default_meson_env())
         with self.dump_logs_on_error("meson-logs/testlog.txt"):
             run_build_system_command(
